@@ -1,65 +1,40 @@
 use super::*;
-use serde::{
-    de::Visitor,
-    de::{self, SeqAccess},
-    ser::SerializeStruct,
-    Deserialize, Serialize,
-};
+use serde::{de::Visitor, ser::SerializeMap, Deserialize, Serialize};
 
 impl<T: Serialize> Serialize for KeyMap<T> {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: ::serde::Serializer,
     {
-        let mut state = serializer.serialize_struct("KeyMap", 1)?;
-        let values = self.iter().collect::<Vec<_>>();
-        state.serialize_field("values", &values)?;
+        let mut state = serializer.serialize_map(self.len().into())?;
+        for (k, v) in self.iter() {
+            state.serialize_entry(&k, v)?;
+        }
         state.end()
     }
 }
 
-struct PhmVisitor<T> {
+struct KeyMapVisitor<T> {
     _m: std::marker::PhantomData<T>,
 }
 
-impl<'de, T: Deserialize<'de>> Visitor<'de> for PhmVisitor<T> {
+impl<'de, T: Deserialize<'de>> Visitor<'de> for KeyMapVisitor<T> {
     type Value = KeyMap<T>;
 
     fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
         formatter.write_str("struct KeyMap")
     }
 
-    fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
-    where
-        A: SeqAccess<'de>,
-    {
-        let mut res = KeyMap::<T>::default();
-        let values: Vec<(Handle, T)> = seq
-            .next_element()?
-            .ok_or_else(|| de::Error::missing_field("values"))?;
-        for (k, v) in values {
-            res.insert(k, v).expect("oom");
-        }
-        Ok(res)
-    }
-
     fn visit_map<A>(self, mut map: A) -> Result<Self::Value, A::Error>
     where
         A: ::serde::de::MapAccess<'de>,
     {
-        let mut values = None;
-        while let Some(key) = map.next_key::<String>()? {
-            match key.as_str() {
-                "values" => {
-                    values = map.next_value()?;
-                }
-                _ => {}
-            }
+        let mut cap = map.size_hint().unwrap_or(128);
+        if !cap.is_power_of_two() {
+            cap = cap.next_power_of_two();
         }
-        let values: Vec<(Handle, T)> = values.ok_or_else(|| de::Error::missing_field("values"))?;
-
-        let mut res = KeyMap::default();
-        for (k, v) in values {
+        let mut res = KeyMap::with_capacity(cap, SysAllocator::default()).expect("oom");
+        while let Some((k, v)) = map.next_entry()? {
             res.insert(k, v).expect("oom");
         }
         Ok(res)
@@ -71,18 +46,19 @@ impl<'de, T: Deserialize<'de>> Deserialize<'de> for KeyMap<T> {
     where
         D: ::serde::Deserializer<'de>,
     {
-        deserializer.deserialize_struct(
-            "KeyMap",
-            &["values"],
-            PhmVisitor {
-                _m: Default::default(),
-            },
-        )
+        deserializer.deserialize_map(KeyMapVisitor {
+            _m: Default::default(),
+        })
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::{
+        compiler::{CaoIr, Lane},
+        prelude::CaoProgram,
+    };
+
     use super::*;
 
     #[test]
@@ -124,4 +100,24 @@ mod tests {
         let res = map2.get(Handle(123)).unwrap();
         assert_eq!(*res, 69);
     }
+
+    #[test]
+    fn can_serialize_program_cbor() {
+        let program = crate::compiler::compile(
+            &CaoIr {
+                lanes: vec![Lane {
+                    name: Some("poggers".into()),
+                    arguments: vec![],
+                    cards: vec![],
+                }],
+            },
+            None,
+        )
+        .unwrap();
+
+        let mut payload = Vec::new();
+        ciborium::ser::into_writer(&program, &mut payload).unwrap();
+
+        let _: CaoProgram = ciborium::de::from_reader(payload.as_slice()).unwrap();
+    }
 }
